; OrbitPlot.pro
;
;  This routine reads two files and produces 'curtain' type plots and difference plots for 
;  each orbit
;
;  **********What follows is the general algorithmic flow of OrbitPlot *****************
;
; 1 - read script inputs (procedure)
; 2 - read input file  (procedure)
; 3 - read geolocation data in he5 files  (procedure)
;  
; Loop over parameters (species)
; 
;   4 - read the data in he5 files  (procedure)
;  
;   Loop over latitude
;
;     5 - find one orbit (get begin/end of orbit ~ -61deg south lat)
;     6 - choose scan type user requested  (procedure)
;     7 - if user wants a pressure grid check if it exists - if it does
;         not exist then interpolate to a pressure grid	 (procedure)
;     8 - do the pressure plotting (procedure)
;     9 - do the altitude plotting (procedure)
;
;   End of looping over latitudes
;
; End of looping over parameters
;
; 10 - Close the postscipt file
;
; End of program
;
;
;*******what follows is a key for the input file**********************
;    line  0: Third Plot flag ( 0:no, 1:yes )
;          1: file 1 (input)
;          2: file 1 label
;          3: file 2 (input)
;          4: file 2 label
;          5: Plot only data w/positive precision (1 = yes, 0 = no) 
;          6: number of species (repeat 7-10 for each species)
;          7: species name (as in file)
;          8: minimum
;          9: maximum
;         10: plot log/linear data scale (1 = log, 0 = linear)
;         11: PS-send to postscript, X-send to window, Z-send to png files
;         12: P or Z for pressure or altitude
;         13: (1 = downscans, 2 = upscans) at -23.5 Az, 0 for both
;             (3 = downscans at -22.4 Az)
;             If file has no azimuth (ie HIR2CLCC) set this scan flag to 0
;         14: filename for output
;  *******************************************************************       
;
;*****following are qualifiers for running OrbitPlot in an X-term environment*****
;
;       -i .input file name
;  	-d minimum value for difference plot
;	-D maximum value for difference plot
;	-t minimum value for temperature plot
;	-T maximum value for temperature plot
;	-r minimum value for radiance plot
;	-R maximum value for radiance plot
;	-m minimum latitude defining an orbit (will be different
;          when plotting LIMS data or MLS L2 files)
;   -p bottom pressure index for vertical plot axis
;	-P top pressure index for vertical plot axis
;   -z bottom altitude index for vertical plot axis *Obsolete - now in routine 
;	-Z top altitude index for vertical plot axis    *AltitudeLimits. CR340
;	-l minimum latitude to plot
;	-L maximum latitude to plot
;	-n1 plot first file only
;      
;---------------------------------------------------------------------------
;The ReadScriptInput procedure
PRO ReadScriptInput, DiffMin, DiffMax, MinLatIn, PBot, PTop, InFile, $
                      RDiffMin, RDiffMax, TDiffMin, TDiffMax, BegLat, EndLat, NDFiles
;  Read input config stuff
;
InFile = 'OrbitPlot_2005d131_L2SIPS.input'

;InFile = 'OrbitPlot.input'

PRINT, 'Enter input config file name '
READ, InFile
PRINT, 'Enter difference plot minimum '
READ, DiffMin
PRINT, 'Enter difference plot maximum '
READ, DiffMax
PRINT, 'Enter temperature difference plot minimum '
READ, TDiffMin
PRINT, 'Enter temperature difference plot maximum '
READ, TDiffMax
PRINT, 'Enter radiance difference plot minimum '
READ, RDiffMin
PRINT, 'Enter radiance difference plot maximum '
READ, RDiffMax
PRINT, 'Enter minimum latitude to determine an orbit '
READ, MinLatIn
PRINT, 'Enter minimum pressure index for plots '
READ, PBot
PRINT, 'Enter maximum pressure index for plots '
READ, PTop
;PRINT, 'Enter minimum altitude index for plots '
;READ, ZBot
;PRINT, 'Enter maximum altitude index for plots '
;READ, ZTop
PRINT, 'Enter minimum of latitude range for plots '
READ, BegLat
PRINT, 'Enter maximum of latitude range for plots '
READ, EndLat
PRINT, 'Enter number of input data files '
READ, NDFiles

END ;ReadScriptInput
;-----------------------------------------------------------------------------

;Read the input file procedure
PRO ReadInputFile,InFile,PBot,PTop,File1,File2,VertPar,NPARS,PARNAMES,PARSTR,  $
                  NORBSIN,NSCAN,FILEDESCRIP1,FILEDESCRIP2,PARMINS,PARMAXS,Logs, $
		  PGridIn1,PGridIn2,PGridIn3,PrecParNames,PrecFlg,File3,FileDescrip3,ThirdFile

Print, 'PBot PTop', PBot, PTop

OPENR, LUN, /GET_LUN, InFile
;
;  Get input files and file descriptions
;
File1 = " "
File2 = " "
FileDescrip1 = " "
FileDescrip2 = " "
READF, LUN, ThirdFile
READF, LUN, File1
READF, LUN, FileDescrip1
READF, LUN, File2
READF, LUN, FileDescrip2
;read in optional 3rd file to replace diff plot
If(ThirdFile eq 1) then begin
  File3 = " "
  FileDescrip3 = " "
;ThirdFile = 0
;If (eof(lun) ne 1) then begin
  READF, LUN, File3
  READF, LUN, FileDescrip3
;   ThirdFile = 1
endif
;
;get option for only plotting data w/positive precisions
Readf, Lun, PrecFlg

;if no hirdls file, don't check for neg precision values
if(PrecFlg eq 1) then begin
   I1 = strpos(File1,'HIR')
   I2 = strpos(File2,'HIR')
   if(I1 lt 0 and I2 lt 0) then begin
	PrecFlg = 0
	print,'***Cannot check for neg precisions - neither file is HIRDLS***'
   endif
endif
;
;  Get number of parameters to plot
;
READF, LUN, NPars
;
;  Get parameter names and min/max values for color bar range
;
ParNames = SINDGEN(NPars)
ParName = " "
ParMins = FINDGEN(NPars)
ParMaxs = FINDGEN(NPars)
if(PrecFlg eq 1) then begin
   PrecParNames = SINDGEN(NPars)
endif
;array of linear/log color scale flags - flag for each product
logs = INDGEN(npars)

FOR IPar = 0, NPars-1 DO BEGIN  
  READF, LUN, ParName
  ParNames(IPar) = ParName
  if(PrecFlg eq 1) then begin
     PrecParNames(Ipar) = ParName + 'Precision'
  endif
  READF, LUN, ParMin
  ParMins(IPar) = ParMin
  READF, LUN, ParMax
  ParMaxs(IPar) = ParMax
  READF, LUN, LinLogFlg   ;read in user chosen lin/log flag for this product
  logs(ipar) = LinLogFlg 
; stop if user's range is le 0 for a log plot
  IF ( (parmins(ipar) le 0) or (parmaxs(ipar) le 0) ) and (logs(ipar) eq 1)  THEN BEGIN
    print, ' Problem with .input file!  Param min and max has to be GT 0 for log plots for ',parnames(ipar)
    stop
  ENDIF
ENDFOR

ParStr = StrCompress(StrUpCase(ParNames), /Remove_All)   ; all uppercase, blanks removed; used to read radiances 
;
;  Give user option to put plots in a postscript file (PS) or to the screen (X) or png (Z)
;
DisplayType = " "
READF, LUN, DisplayType 
;
;  Give user option of pressure (P) or altitude (Z) for vertical coordinate
;
VertPar = " "
READF, LUN, VertPar

If (STRMID(ParStr[0],0,3) EQ 'RAD'  AND  VertPar EQ 'P') Then Begin
  Print
  Print, 'Warning: Plotting Radiances on P grid not supported! Try plotting on Z'
  Stop
EndIF

;
;  The value of NScan here tells which type of scan to plot as follows:
;
;  NScan = 0 refers to up and down scans
;  NScan = 1 refers to down scans at -23.5 Azimuth
;  NScan = 2 refers to up   scans at -23.5 Azimuth
;  NScan = 3 refers to down scans at -22.4 Azimuth
;
NScan = " "
READF, LUN, NScan
OutFileName = " "
READF, LUN, OutFileName 


NOrbsIn = 0

NoSUFlag = 0
PGridIn1 = 0
PGridIn2 = 0
PGRIDIn3 = 0

set_plot, DisplayType

IF DisplayType EQ 'Z' THEN begin   ;png file option
    Device, Z_Buffer = 0
    wsize = [940, 580]
    wsize = [810, 600]
    Device, set_resolution=wsize
    colour_ps  ;; set the color table
    !p.background=255
    !p.color=0
ENDIF

IF DisplayType EQ 'PS' THEN begin
 device,filename=OutFileName
 colour_ps;
endif


END ;ReadInputFile

;---------------------------------------------------------------------------

;The FileLogistics procedure
PRO FileLogistics, File1, File2, File3, File1Type, File2Type, File3Type,  $ 
                   VertPar, ThirdFile
;Check file type logistics  

File1Pos = strpos(File1,'HIR') ;get positions of the filename in the pathname
File2Pos = strpos(File2,'HIR')
If ( ThirdFile eq 1 ) Then Begin
    File3Pos = strpos(File3,'HIR')
Endif

If ( STRUPCASE(STRMID(File1,File1Pos,7)) eq 'HIR2CLC'  ) Then Begin
   File1Type = STRUPCASE(STRMID(File1,File1Pos,8))
Endif Else Begin  
   File1Type = STRUPCASE(STRMID(File1,File1Pos,7))
Endelse

If ( STRUPCASE(STRMID(File2,File2Pos,7)) eq 'HIR2CLC'  ) Then Begin
   File2Type = STRUPCASE(STRMID(File2,File2Pos,8))
Endif Else Begin  
   File2Type = STRUPCASE(STRMID(File2,File2Pos,7))
Endelse


If ( ThirdFile eq 1 ) Then Begin
   If ( STRUPCASE(STRMID(File3,File3Pos,7)) eq 'HIR2CLC'  ) Then Begin
      File3Type = STRUPCASE(STRMID(File3,File3Pos,8))
   Endif Else Begin  
      File3Type = STRUPCASE(STRMID(File3,File3Pos,7))
   Endelse
Endif

; warn user and stop if plotting hirdls2 files on altitude
If (VertPar eq 'Z') Then Begin
   If(  (STRMID(File1Type,0,7) EQ 'HIRDLS2') OR (STRMID(File2Type,0,7) EQ 'HIRDLS2')  ) Then Begin
      Print
      Print, 'Warning: Plotting HIRDLS2 files on altitude not supported, use HIRPROF'
      Stop
   EndIF
   if(ThirdFile eq 1) then begin
      if(STRMID(File3Type,0,7) EQ 'HIRDLS2') then begin
         Print
         Print, 'Warning: Plotting HIRDLS2 files on altitude not supported, use HIRPROF'
         Stop
      endif
   endif
EndIF


END ;FileLogistics

;--------------------------------------------------------------------------------

;Read geolocation data from he5 files
PRO ReadGeolocation, File1, File2, File3, VertPar, PGridIn1, PGridIn2, PGridIn3,   $
                     Pressure1, Pressure2, Pressure3, Lats, Lons, ParNames, ProfID,    $
                     indx1, indx2, indx3, ThirdFile
;If there is a 3rd plot - NO diff plot, but still need to
;check if all three plots are aligned, and
;do need to compute PGrid for pressure plot of 3rd file.

; Read in the times and check for a difference larger than 2 seconds
s=get_aura(File1, 'Time', time1)
s=get_aura(File2, 'Time', time2)
timediff = time1 - time2
m = where (abs(timediff) gt 2)
if(thirdfile eq 1) then begin  
   s=get_aura(File3, 'Time', time3)
   timediff3 = time1 - time3
   m3 = where (abs(timediff3) gt 2)
endif

; If times are off, find the times which are in sync in two files 
; create the indx1 and indx2 list of indices and apply them to all fields read in
; indx1 is list of times in file one
; indx2 is list of times in file two that sync with those in file one
nprofs=-1

if (m[0] ne -1) then begin
  nprof1=size(time1)
  indx1=intarr(nprof1(1))
  indx2=intarr(nprof1(1))
  for i=0,nprof1(1)-1 do begin ; Loop over all times in file 1
    timediff=abs(time2-time1(i))
    minDiff = min(timediff,ix) ; get the index of the minimum difference
    if (abs(time1(i)-time2(ix)) le 2) then begin ; if within 2 seconds then in sync
      nprofs=nprofs+1
      indx1(nprofs)=i
      indx2(nprofs)=ix
    endif
  endfor
endif else begin ; Not out of sync, just create indices for the two files
  nprof1=size(time1)
  indx1=indgen(nprof1(1)) 
  indx2=indgen(nprof1(1)) 
  nprofs=nprof1(1)
endelse

if (nprofs lt 1) then message, 'No times overlap within the 1st two files'

if(thirdfile eq 1) then begin  ;repeat same if blk above for 3rd file
   nprofs=-1

   if (m3[0] ne -1) then begin
     nprof1=size(time1)
     indx1=intarr(nprof1(1))
     indx3=intarr(nprof1(1))
     for i=0,nprof1(1)-1 do begin ; Loop over all times in file 1
       timediff3=abs(time3-time1(i))
       minDiff3 = min(timediff3,ix) ; get the index of the minimum difference
       if (abs(time1(i)-time3(ix)) le 2) then begin ; if within 2 seconds then in sync
         nprofs=nprofs+1
         indx1(nprofs)=i
         indx3(nprofs)=ix
       endif
     endfor
   endif else begin ; Not out of sync, just create indices for the two files
     nprof1=size(time1)
     indx1=indgen(nprof1(1)) 
     indx3=indgen(nprof1(1)) 
     nprofs=nprof1(1)
   endelse

   if (nprofs lt 1) then message, 'No times overlap with the 3rd file'
endif

s=get_aura(File1, 'Latitude', Lats)
IF s EQ -1 THEN s = GET_AURA(File1, 'Latitude', swathname=ParNames(0), Lats) 
if (s ne -1) then lats = lats(indx1)
 
s=get_aura(File1, 'Longitude', Lons)  
IF s EQ -1 THEN s = GET_AURA(File1, 'Longitude', swathname=ParNames(0), Lons)
if (s ne -1) then lons = lons(indx1)
  
s=get_aura(File1, 'ProfileID', ProfID)
if (s ne -1) then profID=profID(indx1)

      
;get 2nd file latitude
;s2=get_aura(File2, 'Latitude', Lats2)
;IF s2 EQ -1 THEN s2 = GET_AURA(File2, 'Latitude', swathname=ParNames(0), Lats2)
 
;compare lats from both files
;L = where (Lats ne Lats2)  ;get indices where array values not equal
;if (L[0] ne -1) then begin  ;oh no, some array values are not equal
;   M = where (Lats[L]-Lats2[L] gt 1.0e-4)  ;get indices where array diffs are too big
;   if (M[0] ne -1) then message, 'Your two data files have different Lats!'
;endif  

IF VertPar EQ 'Z' THEN BEGIN

  s=get_aura(File1, 'Altitude', Alt1)
  IF s EQ -1 THEN s = GET_AURA(File1, 'TangentHeight', Alt1)
  IF s eq -1 THEN Message, 'Z requested but no altitude in file 1 '
  if (s ne -1) then alt1=alt1(indx1)
  
  s=get_aura(File2, 'Altitude', Alt2)
  IF s EQ -1 THEN s = GET_AURA(File2, 'TangentHeight', Alt2)
  IF s eq -1 THEN Message, 'Z requested but no altitude in file 2 '
  if (s ne -1) then alt2=alt2(indx2)

  if(ThirdFile eq 1) then begin
     s=get_aura(File3, 'Altitude', Alt3)
     IF s EQ -1 THEN s = GET_AURA(File3, 'TangentHeight', Alt3)
     IF s eq -1 THEN Message, 'Z requested but no altitude in file 3 '
     if (s ne -1) then alt3=alt3(indx3)
  endif

ENDIF

IF VertPar EQ 'P' THEN BEGIN

  s=get_aura(File1, 'Pressure', Pressure1)
  IF s EQ -1 THEN s = GET_AURA(File1, 'Pressure', swathname=ParNames(0), Pressure1)
  if (s ne -1) then Pressure1=Pressure1(indx1)
  s=get_aura(File2, 'Pressure', Pressure2)
  IF s EQ -1 THEN s = GET_AURA(File2, 'Pressure', swathname=ParNames(0), Pressure2)
  if (s ne -1) then Pressure2=Pressure2(indx2)
  if(ThirdFile eq 1) then begin  
     s=get_aura(File3, 'Pressure', Pressure3)
     IF s EQ -1 THEN s = GET_AURA(File3, 'Pressure', swathname=ParNames(0), Pressure3)
     if (s ne -1) then Pressure3=Pressure3(indx3)
  endif

  NDimsP1 = SIZE(Pressure1, /N_DIMENSIONS)
  NDimsP2 = SIZE(Pressure2, /N_DIMENSIONS)
  if(ThirdFile eq 1) then begin
     NDimsP3 = SIZE(Pressure3, /N_DIMENSIONS)
  endif

; Check whether File1 and File2 and File3 are gridded on Pressure or not

  IF NDimsP1 EQ 1 THEN BEGIN ; on pressure grid
    PGridIn1 = 1
  ENDIF

  IF NDimsP2 EQ 1 THEN BEGIN
    PGridIn2 = 1
  ENDIF

  if(ThirdFile eq 1) then begin
     IF NDimsP3 EQ 1 THEN BEGIN
       PGridIn3 = 1
     ENDIF
  endif

; Check that the pressure grids are the same, if both files have a pressure grid - otherwise stop
; as the program is not set up to handle this case
  IF NDimsP1 NE NDimsP2 THEN Message, 'Pressure array dimensions not compatible in 1st two input files '

  IF NDimsP1 EQ 1 THEN BEGIN

    IF N_ELEMENTS(Pressure1) NE N_ELEMENTS(Pressure2) THEN Message, 'Pressure array #elements not compatible in 1st two input files '
    PGridIn = 1

  ENDIF

  IF (ThirdFile eq 1) then begin
     IF NDimsP1 NE NDimsP3 THEN Message, 'Pressure array dimensions not compatible in input files 1,3 '
     IF NDimsP1 EQ 1 THEN BEGIN
       IF N_ELEMENTS(Pressure1) NE N_ELEMENTS(Pressure3) THEN Message, 'Pressure array #elements not compatible in input files 1,3'
       PGridIn = 1
     ENDIF
  ENDIF

;  IF (PGridIn1 EQ 1 AND PGridIn2 EQ 1) THEN BEGIN
;    ix = where (Pressure1(*) NE Pressure2(*))
;    IF (ix(0) GE 0) THEN message,' Pressure grids in two files do not match -- stopping'
;  ENDIF

ENDIF


END ;ReadGeolocation
;-------------------------------------------------------------------------------------

;read data from the he5 files
PRO ReadHe5Data, ParNames, IPar, ParStr, File1, File2, File3, DiffMin, DiffMax, Param1, Param2, Param3, ParamDiff,  $
                 Units, dTitStr, dTitSym, DiffDatRange, RDiffMin, RDiffMax, TDiffMin, TDiffMax,  $
		 PrecParNames, PrecFlg, File1Type, File2Type, File3Type, ParName1, ParName2, ParName3, readRad, Rad1, Rad2, $
         Rad3, indx1,indx2,indx3,NSCAN, scanupflag, scanupflag2,scanupflag3,ThirdFile

  PRINT, 'Reading parameter "' + STRTRIM(ParNames[IPar]) + '".'

  !P.MULTI = [0,4,3]	;open a new window or page 
  
  IF STRMID(ParStr(IPar),0,3) EQ 'RAD' THEN BEGIN  ;data are radiances

    ChNum = FIX(STRMID(ParStr(IPar),9,2))

    if (readRad eq 1) then begin  ; Only read the radiances the first time
      s=get_aura(File1, 'Radiance', Rad1)
      if (s ne -1) then rad1=rad1(*,*,indx1)
      s=get_aura(File2, 'Radiance', Rad2)
      if (s ne -1) then rad2=rad2(*,*,indx2)
;     notice - indx3 cause did do comparison processing  
;     in ReadGeolocation if 3rd plot
      If ( ThirdFile eq 1 ) Then Begin
         s=get_aura(File3, 'Radiance', Rad3)
         if (s ne -1) then rad3=rad3(*,*,indx3)
      Endif
      print, 'Read all the radiances for all files'
      readRad =0
    endif

    Param1 = REFORM(Rad1(*,ChNum-1,*))
    Param2 = REFORM(Rad2(*,ChNum-1,*))
    If ( ThirdFile eq 1 ) Then Begin
       Param3 = REFORM(Rad3(*,ChNum-1,*))
    Endif

    T1 = where(Param1 eq -999.0)         ;change missing values to Nans
    T2 = where(Param2 eq -999.0)
    if(T1(0) ne -1) then Param1(T1) = !values.F_NAN   
    if(T2(0) ne -1) then Param2(T2) = !values.F_NAN
    If ( ThirdFile eq 1 ) Then Begin
       T3 = where(Param3 eq -999.0)
       if(T3(0) ne -1) then Param3(T3) = !values.F_NAN
    Endif

    Units = ' : '

    ParamDiff = Param1 / Param2
    DiffDatRange = [RDiffMin,RDiffMax]
    dTitStr = 'Ratio: '

    dTitSym = ' / '

       ParName1 = ParNames(ipar)
       ParName2 = ParNames(ipar)
       If ( ThirdFile eq 1 ) Then Begin
          ParName3 = ParNames(ipar)
       Endif

  ENDIF ELSE BEGIN   ;data are mixing ratios

;   following block reads & processes parameters for both data files (CR337) ----------- 

       s1 = 1
       s2 = 1
       s3 = 1
       s=get_aura(File1, ParNames(IPar), Param1)
       if s eq -1 then s1 = -1
       ParName1 = ParNames(ipar)
       s=get_aura(File2, ParNames(IPar), Param2)
       if s eq -1 then s2 = -1
       ParName2 = ParNames(ipar)
       If ( ThirdFile eq 1 ) Then Begin
          s=get_aura(File3, ParNames(IPar), Param3)
          if s eq -1 then s3 = -1
          ParName3 = ParNames(ipar)
       Endif

       ;user asks for parameter, but neither file has it - then stop execution
       if (ThirdFile eq 1 ) Then Begin
          if (s1 eq -1 and s2 eq -1 and s3 eq -1) then begin
            print, '               ***** ',ParNames(ipar), ' is requested, but this *****'
	        message, ' ****** field is in neither file!  ******'
          endif   
       endif else begin 
          if(s1 eq -1 and s2 eq -1) then begin
             print, '               ***** ',ParNames(ipar), ' is requested, but this *****'
	         message, ' ****** field is in neither file!  ******'
          endif
       endelse

       ;what if only one file has H2O18/20, then read H2O instead
       if (s1 eq -1) and (ParNames(ipar) eq 'H2O18') then begin
          s = get_aura(File1,'H2O',Param1)
	      ParName1 = 'H2O'
       endif
       if (s1 eq -1) and (ParNames(ipar) eq 'H2O20') then begin 
          s = get_aura(File1,'H2O',Param1)
	      ParName1 = 'H2O'
       endif
       if (s2 eq -1) and (ParNames(ipar) eq 'H2O18') then begin
          s = get_aura(File2,'H2O',Param2)
	      ParName2 = 'H2O'
       endif
       if (s2 eq -1) and (ParNames(ipar) eq 'H2O20') then begin
          s = get_aura(File2,'H2O',Param2)
	      ParName2 = 'H2O'
       endif
       if (ThirdFile eq 1 ) Then Begin
          if (s3 eq -1) and (ParNames(ipar) eq 'H2O18') then begin
             s = get_aura(File3,'H2O',Param3)
	         ParName3 = 'H2O'
          endif
          if (s3 eq -1) and (ParNames(ipar) eq 'H2O20') then begin
             s = get_aura(File3,'H2O',Param3)
	         ParName3 = 'H2O'
          endif
       endif

;   end of reading/processing parameters ---------------------
    
;   use only middle psi angle for HIR2CLCG files
    IF(File1Type eq 'HIR2CLCG') Then BEGIN
       temp1 = Param1
       SizeParam1 = size(Param1)
       Param1 = FltArr(SizeParam1(1),SizeParam1(3))
       Param1(*,*) = temp1(*,3,*)
    ENDIF
    IF(File2Type eq 'HIR2CLCG') Then BEGIN
       temp2 = Param2
       SizeParam2 = size(Param2)
       Param2 = FltArr(SizeParam2(1),SizeParam2(3))
       Param2(*,*) = temp2(*,3,*)
    ENDIF
    if (ThirdFile eq 1) Then Begin
       IF(File3Type eq 'HIR2CLCG') Then BEGIN
          temp3 = Param3
          SizeParam3 = size(Param3)
          Param3 = FltArr(SizeParam3(1),SizeParam3(3))
          Param3(*,*) = temp3(*,3,*)
       ENDIF
    endif

; only keep the overlapping profiles
    help,param1
    param1=param1(*,indx1)
    param2=param2(*,indx2)

    T1 = where(Param1 eq -999.0)         ;change missing values to Nans
    T2 = where(Param2 eq -999.0)
    if(T1(0) ne -1) then Param1(T1) = !values.F_NAN   
    if(T2(0) ne -1) then Param2(T2) = !values.F_NAN

    if (ThirdFile eq 1) Then Begin
       param3=param3(*,indx3)
       T3 = where(Param3 eq -999.0)
       if(T3(0) ne -1) then Param3(T3) = !values.F_NAN
    endif
    
; Set negative values to NaN - for MLS
    T1 = where(Param1 lt 0.0)
    T2 = where(Param2 lt 0.0)
    if(T1(0) ne -1) then Param1(T1) = !values.F_NAN   
    if(T2(0) ne -1) then Param2(T2) = !values.F_NAN
    if (ThirdFile eq 1) Then Begin
       T3 = where(Param3 lt 0.0)
       if(T3(0) ne -1) then Param3(T3) = !values.F_NAN
    endif

    ;stop processing if any of the mixing ratios are negative
;   if(T1(0) ne -1) then message, 'mixing ratios are negative and not missing for file1'
;   if(T2(0) ne -1) then message, 'mixing ratios are negative and not missing for file2'

;if only plotting data w/positive precisions (and from a hirdls file)
    if ( (PrecFlg eq 1) and (strpos(File1,'HIR') gt -1) ) then begin  ;do 1st file
       s = get_aura(File1,PrecParNames(Ipar),PrecParam1)
       if (s eq 0) then begin ;if reading precisions was successful, chk for neqative precisions
          dim = size( PrecParam1,/dimensions) ;get dimensions
          for i = 0, dim(0) - 1 do begin ; altitude loop
          
             T1 = where(PrecParam1[i,*] lt 0.0)   
             if(T1(0) ne -1) then Param1[i,T1] = !values.F_NAN  ; set corresponding data to NAN

;            Maskout up or down scans from the data arrays if user chooses.
;            This does NOT affect the ChooseScanType subroutine!         
            IF( (NSCAN eq 1) or (NSCAN eq 3) ) then begin ;want to plot downscans
               ScanIndex = where(scanupflag eq 1) ;maskout upscans
                Param1[i,ScanIndex] = !values.F_NAN
             ENDIF
             IF( NSCAN eq 2 ) then begin ; want to plot upscans
                ScanIndex = where(scanupflag eq 0) ;maskout downscans
                Param1[i,ScanIndex] = !values.F_NAN
             ENDIF

          endfor ; altitude loop
;          PrecParam1=PrecParam1(*,indx1)
;          T1 = where(PrecParam1 lt 0.0)
;          if(T1(0) ne -1) then Param1(T1) = !values.F_NAN  ;set corresponding data to NAN
       endif else begin  ;if reading precisions was unsuccessful, warn user and continue
          print, 'File ',File1,' has no precision values!'
       endelse
    endif
    if ( (PrecFlg eq 1) and (strpos(File2,'HIR') gt -1) ) then begin  ;do 2nd file
       s = get_aura(File2,PrecParNames(Ipar),PrecParam2)
       if (s eq 0) then begin ;if reading precisions was successful, chk for neqative precisions
          dim = size( PrecParam2,/dimensions) ;get dimensions
          for i = 0, dim(0) - 1 do begin ; altitude loop
               
             T2 = where(PrecParam2[i,*] lt 0.0)   
             if(T2(0) ne -1) then Param2[i,T2] = !values.F_NAN  ; set corresponding data to NAN

;            Maskout up or down scans from the data arrays if user chooses.
;            This does NOT affect the ChooseScanType subroutine!         
             IF( (NSCAN eq 1) or (NSCAN eq 3) ) then begin ;want to plot downscans
                ScanIndex = where(scanupflag2 eq 1) ;maskout upscans
                Param2[i,ScanIndex] = !values.F_NAN
             ENDIF
             IF( NSCAN eq 2 ) then begin ; want to plot upscans
                ScanIndex = where(scanupflag2 eq 0) ;maskout downscans
                Param2[i,ScanIndex] = !values.F_NAN
             ENDIF
              
          endfor ; altitude loop
;          PrecParam2=PrecParam2(*,indx2)
;          T2 = where(PrecParam2 lt 0.0)
;          if(T2(0) ne -1) then Param2(T2) = !values.F_NAN  ;set corresponding data to NAN
       endif else begin  ;if reading precisions was unsuccessful, warn user and continue
          print, 'File ',File2,' has no precision values!'
       endelse
    endif
; do third file if it exists
  if(thirdfile eq 1) then begin
    if ( (PrecFlg eq 1) and (strpos(File3,'HIR') gt -1) ) then begin  ;do 3rd file
       s = get_aura(File3,PrecParNames(Ipar),PrecParam3)
       if (s eq 0) then begin ;if reading precisions was successful, chk for neqative precisions
          dim = size( PrecParam3,/dimensions) ;get dimensions
          for i = 0, dim(0) - 1 do begin ; altitude loop
               
             T3 = where(PrecParam3[i,*] lt 0.0)   
             if(T3(0) ne -1) then Param3[i,T3] = !values.F_NAN  ; set corresponding data to NAN

;            Maskout up or down scans from the data arrays if user chooses.
;            This does NOT affect the ChooseScanType subroutine!         
             IF( (NSCAN eq 1) or (NSCAN eq 3) ) then begin ;want to plot downscans
                ScanIndex = where(scanupflag3 eq 1) ;maskout upscans
                Param3[i,ScanIndex] = !values.F_NAN
             ENDIF
             IF( NSCAN eq 2 ) then begin ; want to plot upscans
                ScanIndex = where(scanupflag3 eq 0) ;maskout downscans
                Param3[i,ScanIndex] = !values.F_NAN
             ENDIF
              
          endfor ; altitude loop
       endif else begin  ;if reading precisions was unsuccessful, warn user and continue
          print, 'File ',File3,' has no precision values!'
       endelse
    endif
  endif ; if block for third file

;   grab the 1st 3 characters of ParNames
    ParName = '   '
    reads,ParNames(ipar),ParName,format='(a3)'

    IF ParNames(IPar) EQ 'Temperature' THEN BEGIN

      Units = ' [K]: '

      ParamDiff = Param1 - Param2
      DiffDatRange = [TDiffMin,TDiffMax]
      dTitStr = 'Difference [K]: '

    ENDIF ELSE BEGIN

      IF (ParNames(IPar) EQ 'H2O') OR (ParNames(IPar) EQ 'O3') OR $
      (ParNames(IPar) EQ 'H2O18') OR (ParNames(IPar) EQ 'H2O20') OR $
      (ParNames(IPar) EQ 'CH4') OR (ParNames(IPar) EQ 'N2O') THEN BEGIN

        Param1 = 1.0E+6 * Param1
        Param2 = 1.0E+6 * Param2
        if(ThirdFile eq 1) then begin
           Param3 = 1.0E+6 * Param3
        endif
	    Units = ' [ppmv]: '

      ENDIF ELSE IF (ParName eq 'AER') THEN BEGIN
        
        Param1 = 1.0E+9 * Param1
        Param2 = 1.0E+9 * Param2        
        if(ThirdFile eq 1) then begin
           Param3 = 1.0E+9 * Param3
        endif
        Units = ' [(m2/mol)1E9]: '

      ENDIF ELSE IF (ParName eq 'GPH') THEN BEGIN
        Units = 'm'

      ENDIF ELSE BEGIN

        Param1 = 1.0E+9 * Param1
        Param2 = 1.0E+9 * Param2
        if(ThirdFile eq 1) then begin
           Param3 = 1.0E+9 * Param3
        endif
    	Units = ' [ppbv]: '

      ENDELSE

      ParamDiff = 100.0 * ((Param1 - Param2) / Param2)
      DiffDatRange = [DiffMin,DiffMax]
      dTitStr = 'Difference [%]: '

    ENDELSE

    dTitSym = ' - '

  ENDELSE


END ;ReadHe5Data

;----------------------------------------------------------------------------------------

;Read LIMS data if input instead of HE5 data
PRO ReadLIMS, File1, VertPar, PGridIn1, Pres1, LLats, LLons, ParNames, IPar, DiffMin, DiffMax, $
               Param1, Param2, ParamDiff, Units, dTitStr, dTitSym, DiffDatRange, TDiffMin, TDiffMax, $
	       ParName1, ParName2
	       
lims_idl_reader, File1, LData, LSecsInDay, LLats, LLons, LDNFlag, Month, Day

;convert any missing values to Nan
T = where(Ldata eq -999.0)
if (Ldata(T) ne -1) then Ldata(T) = !values.F_NAN

T = where(Ldata lt 0)
if (Ldata(T) ne -1) then message, 'Lims data are negative and not missing'

Pres1 = [0.0010000000,0.0011364600,0.0012915500,0.0014678000,0.0016681000,0.0018957400,0.0021544300,$
        0.0024484400,0.0027825600,0.0031622800,0.0035938100,0.0040842400,0.0046415900,0.0052750000,$
	0.0059948400,0.0068129200,0.0077426400,0.0087992300,0.010000000,0.011364600,0.012915500,$
	0.014678000,0.016681000,0.018957400,0.021544300,0.024484400,0.027825600,0.031622800,$
	0.035938100,0.040842400,0.046415900,0.052750000,0.059948400,0.068129200,0.077426400,$
	0.087992300,0.10000000,0.11364600,0.12915500,0.14678000,0.16681000,0.18957400,0.21544300,$
	0.24484400,0.27825600,0.31622800,0.35938100,0.40842400,0.46415900,0.52750000,0.59948400,$
	0.68129200,0.77426400,0.87992300,1.0000000,1.1364600,1.2915500,1.4678000,1.6681000,1.8957400,$
	2.1544300,2.4484400,2.7825600,3.1622800,3.5938100,4.0842400,4.6415900,5.2750000,5.9948400,$
	6.8129200,7.7426400,8.7992200,10.000000,11.364600,12.915500,14.678000,16.681000,18.957400,$
	21.544300,24.484400,27.825600,31.622800,35.938100,40.842400,46.415900,52.750000,59.948400,$
	68.129200,77.426400,87.992200,100.00000,113.64600,129.15500,146.78000,166.81000,189.57400,$
	215.44300,244.84400,278.25600,316.22800,359.38100,408.42400,464.15900,527.50000,599.48400,$
	681.29200,774.26400,879.92300,1000.0000]

PGridIn1 = 1

IF VertPar EQ 'Z' THEN BEGIN

  Alt1 = LData(*,*,0)
  Alt2 = Alt1
  
ENDIF

PRINT, 'Reading parameter "' + STRTRIM(ParNames[IPar]) + '".'

!P.MULTI = [0,4,3]	;open a new window or page 

IF ParNames(IPar) EQ 'Temperature' THEN BEGIN

  Units = ' [K]: '
  
  Param1 = LData(*,*,1)
  Param2 = Param1

  ParamDiff = Param1 - Param2
  DiffDatRange = [TDiffMin,TDiffMax]
  dTitStr = 'Difference [K]: '
  
ENDIF ELSE BEGIN

  IF (ParNames(IPar) EQ 'CO2') THEN BEGIN
    
    Param1 = LData(*,*,2)
    Param2 = Param1

  ENDIF ELSE IF (ParNames(IPar) EQ 'O3') THEN BEGIN
  
    Param1 = LData(*,*,3)
    Param2 = Param1
  
  ENDIF ELSE IF (ParNames(IPar) EQ 'HNO3') THEN BEGIN
  
    Param1 = LData(*,*,4)
    Param2 = Param1

  ENDIF ELSE IF (ParNames(IPar) EQ 'H2O') THEN BEGIN
  
    Param1 = LData(*,*,5)
    Param2 = Param1

  ENDIF ELSE IF (ParNames(IPar) EQ 'NO2') THEN BEGIN
  
    Param1 = LData(*,*,6)
    Param2 = Param1
    
  ENDIF ELSE BEGIN
  
    Message, 'Invalid LIMS parameter requested'

  ENDELSE
  
  IF (ParNames(IPar) EQ 'H2O') OR (ParNames(IPar) EQ 'O3') OR $
  (ParNames(IPar) EQ 'CH4') OR (ParNames(IPar) EQ 'N2O') THEN BEGIN

    Param1 = 1.0E+6 * Param1
    Param2 = 1.0E+6 * Param2
    Units = ' [ppmv]: '

  ENDIF ELSE BEGIN

    Param1 = 1.0E+9 * Param1
    Param2 = 1.0E+9 * Param2
    Units = ' [ppbv]: '

  ENDELSE

  ParamDiff = 100.0 * ((Param1 - Param2) / Param2)
  DiffDatRange = [DiffMin,DiffMax]
  dTitStr = 'Difference [%]: '

ENDELSE

dTitSym = ' - '

ParName1 = ParNames(Ipar)
ParName2 = parNames(Ipar)

END ;ReadLIMS

;----------------------------------------------------------------------------------------

;set individual top and bottom altitude limits for each species - CR340
PRO  AltitudeLimits, ParName1, ParName2, ZTop, ZBot

If ( ParName1 eq 'Temperature') Then Begin
   ZBot = 0
   ZTop = 80
ENDIF ELSE IF (ParName1 EQ 'H2O') THEN BEGIN
   ZBot = 0
   ZTop = 60
ENDIF ELSE IF (ParName1 EQ 'O3') THEN BEGIN
   ZBot = 0
   ZTop = 70
ENDIF ELSE IF (ParName1 EQ 'NO2') THEN BEGIN
   ZBot = 0
   ZTop = 65
ENDIF ELSE IF ( (ParName1 EQ 'CH4') OR (ParName1 EQ 'N2O')  OR   $
		 (ParName1 EQ 'ClONO2') OR (ParName1 EQ 'N2O5') OR  $
		 (ParName1 EQ 'CFC11') OR (ParName1 EQ 'CFC12') OR  $
		 (ParName1 EQ 'HNO3') ) THEN BEGIN
   ZBot = 0
   ZTop = 50
ENDIF ELSE IF ( strmid(ParName1,0,3) EQ 'AER')   THEN BEGIN
   ZBot = 0
   ZTop = 35
EndIf

END ;AltitudeLimits

;----------------------------------------------------------------------------------------

;choose only scan type user requests
PRO ChooseScanType, NOrbLats, PrevOrbILat, ScanUpFlag, NScan, Azimuth, IDXAll,  $
                    IOrbScanPts, Ilat

       IF ( (NScan EQ 0) OR (Azimuth(Ilat) eq 0 and ScanUpFlag(Ilat) eq 0) ) THEN Begin ;up and down scans together
								;or no scans in either file
	  IDXAll = INDGEN(NOrbLats)   
	  IOrbScanPts = NOrbLats

       ENDIF ELSE BEGIN           ;process either up or down scans
		    
          FOR IAz = 0, NOrbLats-1 DO BEGIN

	    IF ScanUpFlag(PrevOrbILat+IAz) EQ 1 THEN BEGIN

	      IF (NScan EQ 2 AND Azimuth(PrevOrbILat+IAz) LT -46.0) THEN BEGIN

	        IDXAll(IOrbScanPts) = IAz            ;upscans
	        IOrbScanPts = IOrbScanPts + 1

	      ENDIF

            ENDIF ELSE BEGIN     ;downscans

              IF ScanUpFlag(PrevOrbILat+IAz) EQ 0 THEN BEGIN

                IF (NScan EQ 1 AND Azimuth(PrevOrbILat+IAz) LT -46.0) THEN BEGIN

		  IDXAll(IOrbScanPts) = IAz 
	          IOrbScanPts = IOrbScanPts + 1

		ENDIF

		IF (NScan EQ 3 AND Azimuth(PrevOrbILat+IAz) GT -46.0) THEN BEGIN

		  IDXAll(IOrbScanPts) = IAz 
	          IOrbScanPts = IOrbScanPts + 1

		ENDIF

              ENDIF

  	    ENDELSE

          ENDFOR

       ENDELSE          ;end of processing up or down

END ; of ChooseScanType

;-------------------------------------------------------------------------------------

;input or build a pressure grid
PRO PressureGrid, PGridIn, NPres, NOrbLats, ParOrb, ParOut, ILat,  $
		  Pressure, PrevOrbILat, PARNAMES, IPar, PGrid

; This routine determines whether the data is on a pressure grid or not.  If it is not, 
; then interpolation is done to put it on a grid.

	    IF PGridIn EQ 1 THEN BEGIN     ; On a pressure grid, just assign data to output
              
	      ParOut = FLTARR(NPres,NOrbLats)

	      ParOut = ParOrb
	
      	    ENDIF ELSE BEGIN               ; Need to interpolate data to pressure grid
 
              POrb = Pressure(*,PrevOrbILat:ILat-1)
	    
              ParOut = FLTARR(NPres,NOrbLats)

              FOR ILt = 0, NOrbLats-1 DO BEGIN

		PIn = POrb(*,ILt)
		
		ParIn = ParOrb(*,ILt)

;;;; If Pressure log else linear 
;;;; Make sure monotonically increasing with pressure
;;;; Make sure all parameters are positive (see above), and not missing

;; CAC Mods
;; Replaced logic which only kept the positive valued numbers with logic which sets 
;; negative values to NaN.  Needed this fix to not interpolate values in the middle of
;; a profile, rather should keep missing and set negative values to missing.

		APos = WHERE(ParIn GT 0.0, complement=neg, Count)
	     
		IF Count GT 2 then begin  ;need at least 3 levels to interpolate
	     
  		  if (neg[0] ne -1) then ParIn(neg) = !values.F_NAN
	     
		  IF ParNames(IPar) EQ 'Temperature' THEN BEGIN
		    VFND, ALOG(PIn), ParIn, ALOG(PGrid), ParO, cut= !values.F_NAN, linear=1
		    ParOut(*,ILt) = ParO
		  EndIF Else BEGIN
		    VFND, ALOG(PIn), ALOG(ParIn), ALOG(PGrid), ParO, cut= !values.F_NAN, linear=1  ; use ln(VMR) to interpolate
		    ParOut(*,ILt) = EXP(ParO)                                                 ; back to VMR
		  ENDELSE
	     
		ENDIF ELSE BEGIN  ;put Nan values in array
		
		  ParOut(*,ILt) =  !values.F_NAN

		ENDELSE
	     
	      ENDFOR  ;End of NOrbLats loop for pressure gridding
 	      
	    ENDELSE


END ; of PressureGrid

;------------------------------------------------------------------------------

;do plotting on pressure levels
PRO PressurePlot, Log, ParOut1, ParOut2, ParOut3, ParNames, IPar, Units,   $
          FileDescrip1,FileDescrip2, FileDescrip3,ParMins, ParMaxs, IDX, PGrid, LatOrb,   $
		  LonOrb, ParDiffOut, dTitStr, dTitSym, DiffDatRange,   $
		  PTop, PBot, BegLat, EndLat, NDFiles, File1, File2, File3, ProfID,  $
		  ILat,PrevOrbILat, ParName1, ParName2, ThirdFile

; PLOT_XSECTP picks out the section of image from PBot to PTop and 
; plots it on the log of PGrid from PBot to PTop (calls TVPP, which by 
; default sets /YLOG in call to PLOT); so the data are plotted 
; correctly on the y axis (bottom and top of data match those of Y axis)
;
;  Need to call routine to subset latitudes if requested
;
   SubsetLatitude, BegLat, EndLat, LatOrb, IDX, IDXSub1, IDXSub2


 IF Log EQ 0 THEN BEGIN  ;if linear option
  

   PLOT_XSECTP, /scale, REFORM(ParOut1(*,*)), LatOrb, ip=idxSub1, $
     PGrid, PBot=PBot, PTop=PTop, $
     TITLE=ParName1 + Units + FileDescrip1, $
     RANGE=[ParMins(IPar), ParMaxs(IPar)]
   
   IF NDFiles GT 1 THEN BEGIN
  
     PLOT_XSECTP, /scale, REFORM(ParOut2(*,*)), LatOrb, ip=idxSub1, $
       PGrid, PBot=PBot, PTop=PTop, $
       TITLE=ParName2 + Units + FileDescrip2, $
       RANGE=[ParMins(IPar), ParMaxs(IPar)]
     If (ThirdFile eq 1) then begin       
        PLOT_XSECTP, /scale, REFORM(ParOut3(*,*)), LatOrb, ip=idxSub1, $
          PGrid, PBot=PBot, PTop=PTop, $
          TITLE=ParName1 + Units + FileDescrip3, $
          RANGE=[ParMins(IPar), ParMaxs(IPar)]
     Endif else begin  ; diff plot   
        PLOT_XSECTP, /scale, REFORM(ParDiffOut(*,*)), LatOrb, $
          ip=idxSub1, PGrid, PBot=PBot, PTop=PTop, $
          TITLE=dTitStr + FileDescrip1 + dTitSym + FileDescrip2, $
          RANGE=DiffDatRange
     Endelse

   ENDIF

   PLOT_XTRACK, _EXTRA=EXTRA, LatOrb, LonOrb, ip=idxSub1, ProfID,ILat,PrevOrbILat
   PAUSE
   
   IF IDXSub2[0] NE -1 THEN BEGIN
  
     PLOT_XSECTP, /scale, REFORM(ParOut1(*,*)), LatOrb, ip=idxSub2, $
       PGrid, PBot=PBot, PTop=PTop, $
       TITLE=ParName1 + Units + FileDescrip1, $
       RANGE=[ParMins(IPar), ParMaxs(IPar)]
   
     IF NDFiles GT 1 THEN BEGIN
  
        PLOT_XSECTP, /scale, REFORM(ParOut2(*,*)), LatOrb, ip=idxSub2, $
	      PGrid, PBot=PBot, PTop=PTop, $
	      TITLE=ParName2 + Units + FileDescrip2, $
	      RANGE=[ParMins(IPar), ParMaxs(IPar)]

        If (ThirdFile eq 1) then begin       
           PLOT_XSECTP, /scale, REFORM(ParOut3(*,*)), LatOrb, ip=idxSub2, $
             PGrid, PBot=PBot, PTop=PTop, $
             TITLE=ParName1 + Units + FileDescrip3, $
             RANGE=[ParMins(IPar), ParMaxs(IPar)]
        Endif else begin  ; diff plot   	 
           PLOT_XSECTP, /scale, REFORM(ParDiffOut(*,*)), LatOrb, $
	         ip=idxSub2, PGrid, PBot=PBot, PTop=PTop, $
	         TITLE=dTitStr + FileDescrip1 + dTitSym + FileDescrip2, $
	         RANGE=DiffDatRange 
	    ENDELSE

     ENDIF      
     
     PLOT_XTRACK, _EXTRA=EXTRA, LatOrb, LonOrb, ip=idxSub2, ProfID,ILat,PrevOrbILat
     PAUSE   

   ENDIF 

;  put full pathname and file name of the two files at top of plot sheet
   xyouts, 0.0, 1.02, File1, charsize=0.5, /normal
   xyouts, 0.0, 1.011, File2, charsize=0.5, /normal
   if (ThirdFile eq 1) then begin
      xyouts, 0.0, 1.002, File3, charsize=0.5, /normal
   endif

 ENDIF ELSE BEGIN   ;else do log option

     Units = 'log '
     PLOT_XSECTP, /log10, /scale, REFORM(ParOut1(*,*)), LatOrb, $
       ip=idxSub1, PGrid, pbot=pbot, ptop=ptop, $
       TITLE=Units + ParName1 + ' : ' + FileDescrip1, $
       RANGE=[ParMins(IPar), ParMaxs(IPar)]
     
     IF NDFiles GT 1 THEN BEGIN
  
       PLOT_XSECTP, /log10, /scale, REFORM(ParOut2(*,*)), LatOrb, $
	 ip=idxSub1, PGrid, pbot=pbot, ptop=ptop, $
	 TITLE=Units + ParName2 + ' : ' + FileDescrip2, $
	 RANGE=[ParMins(IPar), ParMaxs(IPar)]
     If (ThirdFile eq 1) then begin
        PLOT_XSECTP, /log10, /scale, REFORM(ParOut3(*,*)), LatOrb, $
          ip=idxSub1, PGrid, pbot=pbot, ptop=ptop, $
          TITLE=Units + ParName1 + ' : ' + FileDescrip3, $
          RANGE=[ParMins(IPar), ParMaxs(IPar)]
     endif else begin   ; diff plot
        PLOT_XSECTP, /scale, REFORM(ParDiffOut(*,*)), LatOrb, $
	      ip=idxSub1, PGrid, pbot=pbot, ptop=ptop, $
	      TITLE=dTitStr + FileDescrip1 + dTitSym + FileDescrip2, $
	      RANGE=DiffDatRange
     endelse

     ENDIF
   
     PLOT_XTRACK, _EXTRA=EXTRA, LatOrb, LonOrb, ip=idxSub1, ProfID,ILat,PrevOrbILat
     PAUSE
     
     IF IDXSub2[0] NE -1 THEN BEGIN
    
       PLOT_XSECTP, /log10, /scale, REFORM(ParOut1(*,*)), LatOrb, $
	 ip=idxSub2, PGrid, pbot=pbot, ptop=ptop, $
	 TITLE=ParName1 + Units + FileDescrip1, $
	 RANGE=[ParMins(IPar), ParMaxs(IPar)]
	 
       IF NDFiles GT 1 THEN BEGIN
  
	      PLOT_XSECTP, /log10, /scale, REFORM(ParOut2(*,*)), LatOrb, $
	        ip=idxSub2, PGrid, pbot=pbot, ptop=ptop, $
	        TITLE=ParName2 + Units + FileDescrip2, $
	        RANGE=[ParMins(IPar), ParMaxs(IPar)]
          if(ThirdFile eq 1) then begin
             PLOT_XSECTP, /log10, /scale, REFORM(ParOut3(*,*)), LatOrb, $
	           ip=idxSub2, PGrid, pbot=pbot, ptop=ptop, $
	           TITLE=ParName1 + Units + FileDescrip3, $
	           RANGE=[ParMins(IPar), ParMaxs(IPar)]
          endif else begin   ;  diff plot
	         PLOT_XSECTP, /scale, REFORM(ParDiffOut(*,*)), LatOrb, $
	           ip=idxSub2, PGrid, pbot=pbot, ptop=ptop, $
	           TITLE=dTitStr + FileDescrip1 + dTitSym + FileDescrip2, $
	           RANGE=DiffDatRange
          endelse

       ENDIF

       PLOT_XTRACK, _EXTRA=EXTRA, LatOrb, LonOrb, ip=idxSub2, ProfID,ILat,PrevOrbILat
       PAUSE
     
     ENDIF   

;  put full pathname and file name of the two files at top of plot sheet
   xyouts, 0.0, 1.02, File1, charsize=0.5, /normal
   xyouts, 0.0, 1.011, File2, charsize=0.5, /normal
   if( ThirdFile eq 1) then begin
      xyouts, 0.0, 1.002, File3, charsize=0.5, /normal
   endif

 ENDELSE  ;end of if block for linear or log


END ; of PressurePlot

;-----------------------------------------------------------------------------------

;do plotting on altitude levels
PRO AltitudePlot, Log, ParOrb1, ParOrb2, ParOrb3, LatOrb, LonOrb, IDX, Zbot, Ztop,  $
                  ParNames, IPar, Units, FileDescrip1, FileDescrip2, FileDescrip3,  $
		  ParMins, ParMaxs, DiffDatRange, ParDiffOrb, dTitStr, $
		  dTitSym, BegLat, EndLat, NDFiles, File1, File2, File3, ProfID,  $
		  ILat,PrevOrbILat, ParName1, ParName2, ThirdFile
;
;  Need to call routine to subset latitudes if requested
;
   SubsetLatitude, BegLat, EndLat, LatOrb, IDX, IDXSub1, IDXSub2
;
;  Have a choice whether to plot on log data scale
;
   IF Log EQ 0 THEN BEGIN   ;if linear option
   
     PLOT_XSECT2, /scale, REFORM(ParOrb1(*,*)), LatOrb, $
       ip=idxSub1, zbot=zbot, ztop=ztop, $
       TITLE=ParName1 + Units + FileDescrip1, $
       RANGE=[ParMins(IPar), ParMaxs(IPar)]
     
     IF NDFiles GT 1 THEN BEGIN
  
       PLOT_XSECT2, /scale, REFORM(ParOrb2(*,*)), LatOrb, $
	 ip=idxSub1, zbot=zbot, ztop=ztop, $
	 TITLE=ParName2 + Units + FileDescrip2, $
	 RANGE=[ParMins(IPar), ParMaxs(IPar)]

        If (ThirdFile eq 1) then begin
           PLOT_XSECT2, /scale, REFORM(ParOrb3(*,*)), LatOrb, $
             ip=idxSub1, zbot=zbot, ztop=ztop, $
             TITLE=ParName1 + Units + FileDescrip3, $
             RANGE=[ParMins(IPar), ParMaxs(IPar)]
        endif else begin    ;  diff plot
           PLOT_XSECT2, /scale, REFORM(ParDiffOrb(*,*)), LatOrb, $
	         ip=idxSub1, zbot=zbot, ztop=ztop, $
	         TITLE=dTitStr + FileDescrip1 + dTitSym + FileDescrip2, $
	         RANGE=DiffDatRange
        endelse

     ENDIF
   
     PLOT_XTRACK, _EXTRA=EXTRA, LatOrb, LonOrb, ip=idxSub1, ProfID,ILat,PrevOrbILat
     PAUSE
   
     IF IDXSub2[0] NE -1 THEN BEGIN
    
       PLOT_XSECT2, /scale, REFORM(ParOrb1(*,*)), LatOrb, $
	 ip=idxSub2, zbot=zbot, ztop=ztop, $
	 TITLE=ParName1 + Units + FileDescrip1, $
	 RANGE=[ParMins(IPar), ParMaxs(IPar)]
     
       IF NDFiles GT 1 THEN BEGIN
  
	      PLOT_XSECT2, /scale, REFORM(ParOrb2(*,*)), LatOrb, $
	        ip=idxSub2, zbot=zbot, ztop=ztop, $
	        TITLE=ParName2 + Units + FileDescrip2, $
	        RANGE=[ParMins(IPar), ParMaxs(IPar)]

          if (ThirdFile eq 1) then begin
             PLOT_XSECT2, /scale, REFORM(ParOrb3(*,*)), LatOrb, $
	           ip=idxSub2, zbot=zbot, ztop=ztop, $
	           TITLE=ParName1 + Units + FileDescrip3, $
	           RANGE=[ParMins(IPar), ParMaxs(IPar)]
          endif else begin   ;  diff plot
	         PLOT_XSECT2, /scale, REFORM(ParDiffOrb(*,*)), LatOrb, $
	           ip=idxSub2, zbot=zbot, ztop=ztop, $
	           TITLE=dTitStr + FileDescrip1 + dTitSym + FileDescrip2, $
	           RANGE=DiffDatRange
          endelse
	   
       ENDIF
     
       PLOT_XTRACK, _EXTRA=EXTRA, LatOrb, LonOrb, ip=idxSub2, ProfID,ILat,PrevOrbILat
       PAUSE
     
     ENDIF   

;    put full pathname and file name of the two files at top of plot sheet
     xyouts, 0.0, 1.02, File1, charsize=0.5, /normal
     xyouts, 0.0, 1.011, File2, charsize=0.5, /normal
     if(ThirdFile eq 1) then begin
        xyouts, 0.0, 1.002, File3, charsize=0.5, /normal
     endif

   ENDIF ELSE BEGIN    ;else log option

     Units = 'log '
     PLOT_XSECT2, /log10, /scale, REFORM(ParOrb1(*,*)), LatOrb, $
       ip=idxSub1, zbot=zbot, ztop=ztop, $
       TITLE=Units + ParName1 + ' : ' + FileDescrip1, $
       RANGE=[ParMins(IPar), ParMaxs(IPar)]
     
     IF NDFiles GT 1 THEN BEGIN
  
        PLOT_XSECT2, /log10, /scale, REFORM(ParOrb2(*,*)), LatOrb, $
	      ip=idxSub1, zbot=zbot, ztop=ztop, $
	      TITLE=Units + ParName2 + ' : ' + FileDescrip2, $
	      RANGE=[ParMins(IPar), ParMaxs(IPar)]

        if(ThirdFile eq 1) then begin
           PLOT_XSECT2, /log10, /scale, REFORM(ParOrb3(*,*)), LatOrb, $
             ip=idxSub1, zbot=zbot, ztop=ztop, $
             TITLE=Units + ParName1 + ' : ' + FileDescrip3, $
             RANGE=[ParMins(IPar), ParMaxs(IPar)]
        endif else begin    ;   diff plot
           PLOT_XSECT2, /scale, REFORM(ParDiffOrb(*,*)), LatOrb, $
	         ip=idxSub1, zbot=zbot, ztop=ztop, $
	         TITLE=dTitStr + FileDescrip1 + dTitSym + FileDescrip2, $
	         RANGE=DiffDatRange
        endelse

     ENDIF
   
     PLOT_XTRACK, _EXTRA=EXTRA, LatOrb, LonOrb, ip=idxSub1, ProfID,ILat,PrevOrbILat
     PAUSE
     
     IF IDXSub2[0] NE -1 THEN BEGIN
    
       PLOT_XSECT2, /log10, /scale, REFORM(ParOrb1(*,*)), LatOrb, $
	 ip=idxSub2, zbot=zbot, ztop=ztop, $
	 TITLE=ParName1 + Units + FileDescrip1, $
	 RANGE=[ParMins(IPar), ParMaxs(IPar)]
	 
       IF NDFiles GT 1 THEN BEGIN
  
	      PLOT_XSECT2, /log10, /scale, REFORM(ParOrb2(*,*)), LatOrb, $
	        ip=idxSub2, zbot=zbot, ztop=ztop, $
	        TITLE=ParName2 + Units + FileDescrip2, $
	        RANGE=[ParMins(IPar), ParMaxs(IPar)]

          if(ThirdFile eq 1) then begin
             PLOT_XSECT2, /log10, /scale, REFORM(ParOrb3(*,*)), LatOrb, $
	           ip=idxSub2, zbot=zbot, ztop=ztop, $
	           TITLE=ParName1 + Units + FileDescrip3, $
	           RANGE=[ParMins(IPar), ParMaxs(IPar)]
          endif else begin   ;  diff plot
	         PLOT_XSECT2, /scale, REFORM(ParDiffOrb(*,*)), LatOrb, $
	           ip=idxSub2, zbot=zbot, ztop=ztop, $
	           TITLE=dTitStr + FileDescrip1 + dTitSym + FileDescrip2, $
	           RANGE=DiffDatRange
          endelse

       ENDIF

       PLOT_XTRACK, _EXTRA=EXTRA, LatOrb, LonOrb, ip=idxSub2, ProfID,ILat,PrevOrbILat
       PAUSE
     
     ENDIF   

;    put full pathname and file name of the two files at top of plot sheet
     xyouts, 0.0, 1.02, File1, charsize=0.5, /normal
     xyouts, 0.0, 1.011, File2, charsize=0.5, /normal
     if(ThirdFile eq 1) then begin
        xyouts, 0.0, 1.002, File3, charsize=0.5, /normal
     endif

   ENDELSE

END ; of AltitudePlot

;------------------------------------------------------------------------------------

PRO SubsetLatitude, BegLat, EndLat, LatOrb, IDX, IDXSub1, IDXSub2

;
; First, need to pick out a subset of latitudes to plot if requested
; 
   ILatsP2 = -1
   IDXSub2 = -1
;
; Check for other than default values for latitude range
;
   IF BegLat GT -91.0 AND EndLat LT 91.0 THEN BEGIN
   
     ILats = WHERE(LatOrb(IDX) GE BegLat AND LatOrb(IDX) LE EndLat)
          
     IF ILats[0] NE -1 THEN BEGIN
       
       Index = INDGEN(N_ELEMENTS(ILats))
       NodeInd = WHERE(ILats(Index+1)-ILats GT 1)
       
       IF NodeInd NE -1 THEN BEGIN
       
         ILatsP = ILats(0:NodeInd)
	 ILatsP2 = ILats(NodeInd+1:*) 
	 
       ENDIF ELSE BEGIN
         
	 ILatsP = ILats

       ENDELSE

       IDXSub1 = IDX(ILatsP)
       IF ILatsP2[0] NE -1 THEN IDXSub2 = IDX(ILatsP2)
       print, 'ilatsP', ilatsP
       print, 'ilatsP2', ilatsP2
       
     ENDIF ELSE BEGIN
     
       IDXSub1 = IDX

     ENDELSE

   ENDIF ELSE BEGIN
   
     IDXSub1 = IDX

   ENDELSE    

END ;SubsetLatitude

;------------------------------------------------------------------------------------

PRO PLOT_XTRACK, _EXTRA=EXTRA, lats, lons, ip=ip, NOMAP=NOMAP, ProfID,ILat,PrevOrbILat

IF N_ELEMENTS(NOMAP) EQ 0 THEN NOMAP = 0
xlats = lats(ip)
xlons = lons(ip)

;store 1st/last of subsetted profile IDs of current orbit
beg_ProfID = ProfID(PrevOrbILat)   ;this is index Joe is using 
end_ProfID = ProfID(ILat-1) ;same as above - everywhere in code ilat-1 is the current index


IF NOT(NOMAP) THEN MAP_SET, /CYL, /ADVANCE, /NOBORDER, $
  E_HORIZ={fill:1, color:60}, E_CONT={fill:1, color:120}, color=60, $
  xmargin=5, ymargin=2
OPLOT, xlons, xlats, psym=1, col=200

;    print subsetted profileIDs under the orbit subplot
string_label = 'ProfileID' + string(beg_ProfID) + ':' + string(end_ProfID)
; No /normal in the xyouts so that the text appears under a specific subplot,
; notice how the coordinates are not normalized, but are in data space
     xyouts, -150.0, -90.0, string_label, charsize=0.75 


RETURN
END   ; PLOT_XTRACK

;-------------------------------------------------------------------------------------
;this routine is only needed when png files are chosen (needed for SIPS)
pro genimg_2, filename=filename

if not KEYWORD_SET (filename) then filename = "image.png"
   ;; create image
      image = tvrd()
      tvlct, r, g, b, /get
      thisImage = bytscl(image)
      s = Size(thisImage)
      image3d = bytarr(3,s[1],s[2])
      image3d[0,*,*] = r[Image]
      image3d[1,*,*] = g[Image]
      image3d[2,*,*] = b[Image]
      write_png,filename,image3d
return
end

;---------------------------------------------------------------------------------------

;**************IMPORTANT - The following PRO line is uncommented by the SIPS ******************
;**************************delivery script because SIPS needs OrbitPlot run as a routine!!!!***
;PRO OrbitPlot

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; MAIN ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; VARIABLES
; ParNames - array of fieldnames
; ParName1/2 - replaces ParNames during output(writing plot lables) phase
;              so each plot (per file) can have a different fieldname label
;
;Read Script file input

readRad = 1

ReadScriptInput, DiffMin, DiffMax, MinLatIn, PBot, PTop, InFile, $
                  RDiffMin, RDiffMax, TDiffMin, TDiffMax, BegLat, EndLat, NDFiles

;Read the input file
ReadInputFile,InFile,PBot,PTop,File1,File2,VertPar,NPARS,PARNAMES,PARSTR,  $
              NORBSIN,NSCAN,FILEDESCRIP1,FILEDESCRIP2,PARMINS,PARMAXS,Logs, $
	      PGridIn1,PGridIn2,PGridIn3,PrecParNames,PrecFlg,File3,FileDescrip3,ThirdFile

;Check file type logistics
FileLogistics, File1, File2, File3, File1Type, File2Type, File3Type,   $
               VertPar, ThirdFile

;; Auto title
;; Implemented for SIPS runs, but can be used manually as well
IF (FILEDESCRIP1 eq 'AUTOTITLE') then begin
 fdate1 = strmid(File1, strpos(File1, '.he5')-8, 8)
 year = strmid(fdate1, 2, 2)
 doy  = strmid(fdate1, 5,8)
 iyear = fix(year)
 idoy  = fix(doy)
 ydn2md, iyear, idoy, month, iday
 fdate1 = strtrim(month,2)+'/'+strtrim(iday,2) +'/'+year
 v1 = strpos(file1,'_')
 v2 = strpos(file1,'-c')
 vers = strmid(file1,v1+1,v2-v1-1)
 if(NSCAN eq '1')then begin
    scantype = 'DN'
 endif else if(NSCAN eq '2')then begin
    scantype = 'UP'
 endif else if(NSCAN eq '0')then begin
    scantype = 'DU'
 endif
 FILEDESCRIP1 = vers  + ' ' + scantype + " " + fdate1
ENDIF
;; end of Auto Title

LIMSIn = STRPOS(File1, 'LIMS')
s = 0

IF (LIMSIn EQ -1) THEN BEGIN
  
;Read geolocation data from he5 files (don't do comparison processing for 3rd file)
  ReadGeolocation, File1, File2, File3, VertPar, PGridIn1, PGridIn2, PGridIn3,   $
             Pressure1, Pressure2, Pressure3, Lats, Lons, ParNames, ProfID,  $
             indx1, indx2, indx3, ThirdFile

  If VertPar eq 'P' then begin
;    use only middle psi angle for HIR2CLCG files
     IF(File1Type eq 'HIR2CLCG') Then BEGIN
        temp1 = Pressure1
        SizePressure1 = size(Pressure1)
        Pressure1 = FltArr(SizePressure1(1),SizePressure1(3))
        Pressure1(*,*) = temp1(*,3,*)
     ENDIF
     IF(File2Type eq 'HIR2CLCG') Then BEGIN
        temp2 = Pressure2
        SizePressure2 = size(Pressure2)
        Pressure2 = FltArr(SizePressure2(1),SizePressure2(3))
        Pressure2(*,*) = temp2(*,3,*)
     ENDIF
  ENDIF ;end of VertPar eq P

  s=get_aura(File1, 'ScanAzimuthAtNominalAltitude', Azimuth)  
  if (s ne -1) then Azimuth=Azimuth(indx1)
  s=get_aura(File1, 'ScanUpFlag', ScanUpFlag)
  if (s ne -1) then ScanUpFlag=ScanUpFlag(indx1)
  s2=get_aura(File2, 'ScanUpFlag', ScanUpFlag2)
  if (s2 ne -1) then ScanUpFlag2=ScanUpFlag2(indx2)
  if(ThirdFile eq 1) then begin
     s3=get_aura(File3, 'ScanUpFlag', ScanUpFlag3)
     if (s3 ne -1) then ScanUpFlag3=ScanUpFlag3(indx3)
  endif

ENDIF ELSE BEGIN
;
; Call LIMS data reader to get header information
;
  
  ReadLIMS, File1, VertPar, PGridIn1, Pressure1, LLats, LLons, ParNames, 0, DiffMin, DiffMax, $
            Param1, Param2, ParamDiff, Units, dTitStr, dTitSym, DiffDatRange, TDiffMin, TDiffMax, $
	    ParName1, ParName2
  
  Lats = LLats
  Lons = LLons
  PGridIn1 = 1   ; LIMS must be the first file (from LIMSIn assigment above)

  
ENDELSE

IF (s eq -1 OR LIMSIn GE 0) THEN BEGIN	;make sure s is acceptable if 1st file has no scanup/Az
  
  NoSUFlag = 1

  ScanUpFlag = INTARR(N_ELEMENTS(Lats))
  ScanUpFlag(*) = 0
  Azimuth = INTARR(N_ELEMENTS(Lats))
  Azimuth(*) = 0
  
ENDIF  

IF s2 eq -1 THEN BEGIN	;make sure s2 is acceptable if 2nd file has no scanup/Azimuth
  
  NoSUFlag = 1

  ScanUpFlag2 = INTARR(N_ELEMENTS(Lats))
  ScanUpFlag2(*) = 0
  
ENDIF

if(ThirdFile eq 1) then begin
   IF s3 eq -1 THEN BEGIN	;make sure s3 is acceptable if 3rd file has no scanup/Azimuth
  
     NoSUFlag = 1

     ScanUpFlag3 = INTARR(N_ELEMENTS(Lats))
     ScanUpFlag3(*) = 0
  
   ENDIF
endif

; if 1st file has no SU flag and 2nd file does, ask user to reverse order of files.
; this will prevent mls file w/no SU flag from being 1st file
if (s eq -1 and s2 ne -1) then message,	  $
   '1st file has no scanupflag, but 2nd file does, so reverse input order of data files'
if (ThirdFile eq 1 ) then begin
   if(s eq -1 and s3 ne -1) then message,  $
   '1st file has no scanupflag, but 3rd file does, so reverse input order of 1st and 3rd files'
endif

;if all files have SU, then make sure all have same SU
if(ThirdFile ne 1)then begin     ; two files to check
    if(s ne -1 and s2 ne -1)then begin   ; check only if both have scanup 
      ;now to compare scanupflags from data files
      w = where (ScanUpFlag ne ScanUpFlag2)  ;get indices where arrays not equal
      if (w[0] ne -1) then message, 'Your 1st and 2nd data files differ in scanupflags!'  
    endif
endif else begin   ;  three files to check   
      if(s ne -1 and s2 ne -1 and s3 ne -1) then begin  ;  all 3 files have scanup
         ;now to compare scanupflags from data files
         w = where (ScanUpFlag ne ScanUpFlag2)  ;get indices where arrays not equal
         if (w[0] ne -1) then message, 'Your 1st and 2nd data files differ in scanupflags!'  
         w3 = where (ScanUpFlag ne ScanUpFlag3)  ;get indices where arrays not equal
         if (w3[0] ne -1) then message, '1st and 3rd data files differ in scanupflags!'  
      endif
endelse

;if all files have no scanflags, process with both up and down scans
if (ThirdFile eq 1) then begin    ;   3 files
   if ( s eq -1 and s2 eq -1 and s3 eq -1 ) then begin
     print, 'All 3 files have NO scanupflags, so processing both up & down'
     NSCAN = 0
   endif
endif else begin   ;  2 files
   if ( s eq -1 and s2 eq -1) then begin
     print, 'Both files have NO scanupflags, so processing both up & down'
     NSCAN = 0
   endif
endelse

;ZBot = 6 & ZTop = 65
;PBot = 0 & PTop = 96       ;; indecis of 1000.0 hPa and 0.1 hPa in the pressure array, PGrid 
;PBot = 0 & PTop = 24       ;; indecis of 1000.0 hPa and 0.1 hPa in the pressure array, PGrid 

NLats = N_ELEMENTS(Lats)

MinLat = -90.0

IF (PGridIn1 EQ 0 AND PGridIn2 EQ 0) THEN BEGIN  ; If neither file has pressure grid, calculate one
   PGrid = 1000.0*10^(-1.0*(findgen(145))/24.0)
ENDIF ELSE BEGIN
  IF (PGridIn1 EQ 1) THEN BEGIN    ; Assign pressure grid (already have checked that they match if both files have pressure grid)
   PGrid = Pressure1
  ENDIF ELSE BEGIN
   PGrid = Pressure2
  ENDELSE
ENDELSE

NPres = N_ELEMENTS(PGrid)

IF PTop GT NPres-1 THEN PTop = NPres-1

!X.MARGIN=[21,3]
!P.MULTI = [0,4,3]

;save the original file description because now with CR339, the file description
;after species on top of each subplot will change for ea. species 
Orig_FileDescrip1 = FileDescrip1
Orig_FileDescrip2 = FileDescrip2
if(ThirdFile eq 1) then begin
   Orig_FileDescrip3 = FileDescrip3
endif

FOR IPar = 0, NPars-1 DO BEGIN
;
; First read the data for this parameter
;
;
; Call LIMS data reader if file is LIMS
;
  IF (LIMSIn GE 0) THEN BEGIN
  
    ReadLIMS, File1, VertPar, PGridIn1, Pres1, LLats, LLons, ParNames, IPar, DiffMin, DiffMax, $
              Param1, Param2, ParamDiff, Units, dTitStr, dTitSym, DiffDatRange, TDiffMin, TDiffMax, $
	      ParName1, ParName2

  ENDIF ELSE BEGIN 
  
; read data from the he5 files
    ReadHe5Data, ParNames, IPar, ParStr, File1, File2, File3, DiffMin, DiffMax, Param1, Param2, Param3, ParamDiff,  $
                 Units, dTitStr, dTitSym, DiffDatRange, RDiffMin, RDiffMax, TDiffMin, TDiffMax,  $
		 PrecParNames, PrecFlg, File1Type, File2Type, File3Type, ParName1, ParName2, ParName3, readRad,  $
         Rad1, Rad2, Rad3, indx1, indx2, indx3,NSCAN, scanupflag, scanupflag2, scanupflag3, ThirdFile
  
  ENDELSE


;set individual top and bottom altitude limits for each species - CR340
  AltitudeLimits, ParName1, ParName2, Ztop, ZBot
;
;  Set page arrangement for plot of only one file
;
   IF NDFiles EQ 1 THEN !P.MULTI = [0,2,1]
; 
;  Now begin finding orbits 
;
  PrevOrbILat = 0

  NoMin = 0

  NOrbs = 0

 ;; Need to zero out the orbit label (for png file option)
 previous_orbit = 0
	
  FOR ILat = 0, NLats-1 DO BEGIN
  
;    PRINT, Lats(ILat)
;    print, NLats
    
    IF Lats(ILat) LT -90 THEN CONTINUE

    IF Lats(ILat) GT 0.0 THEN NoMin = 0

    IF (Lats(ILat) LT MinLatIn AND NoMin EQ 0 )THEN BEGIN

      NoMin = 1
      LatWin = ILat + 50
      IF LatWin GT NLats-1 THEN LatWin = NLats - 1
      MinLat = Min(Lats(ILat:LatWin))
      
    ENDIF

    IF (Lats(ILat) EQ MinLat OR ILat EQ NLats-1) THEN BEGIN  ;current orbit

      NOrbs = NOrbs + 1

      ;IF (NOrbs GE 2 AND NOrbs LE 4) THEN BEGIN
      IF (NOrbsIn EQ 0 OR NOrbs LE NOrbsIn) THEN BEGIN	;if a valid orbit
      
        print, 'NOrbs, NOrbsIn ', NOrbs, NOrbsIn

        PRINT, 'Plotting orbit ' + STRTRIM(STRING(NOrbs), 1) + $
	  ', for indices ' + STRTRIM(STRING(PrevOrbILat), 1) + $
	  ' to ' + STRTRIM(STRING(ILat - 1), 1) + '.'

        NOrbLats = ILat - PrevOrbILat

        IF NOrbLats LE 1 THEN PRINT, 'Not enough points in this ' + $
	  'orbit; including them in the next one.'

        IF NOrbLats GT 1 THEN BEGIN  ;enough points in orbit to plot

          IDXAll = INTARR(NOrbLats)

          IOrbScanPts = 0

;         choose only scan type user requested
          ChooseScanType, NOrbLats, PrevOrbILat, ScanUpFlag, NScan, Azimuth, IDXAll,   $
	                  IOrbScanPts, Ilat
          
          IDX = INTARR(IOrbScanPts)
	  IDX = IDXAll(0:IOrbScanPts-1)

          ;PRINT, IDX

          ParOrb1 = Param1(*,PrevOrbILat:ILat-1)
          ParOrb2 = Param2(*,PrevOrbILat:ILat-1)
          if(ThirdFile eq 1) then begin
             ParOrb3 = Param3(*,PrevOrbILat:ILat-1)
          endif
          LatOrb = Lats(PrevOrbILat:ILat-1)
          LonOrb = Lons(PrevOrbILat:ILat-1)

          ParDiffOrb = ParamDiff(*,PrevOrbILat:ILat-1)
;
;  If pressure is vertical axis then put two data sets on same pressure grid and take difference
;
          IF VertPar EQ 'P' THEN BEGIN

;           Input or build a Pressure grid
            PressureGrid, PGridIn1, NPres, NOrbLats, ParOrb1, ParOut1, ILat,  $
			  Pressure1, PrevOrbILat, PARNAMES, IPar, PGrid
	    
            PressureGrid, PGridIn2, NPres, NOrbLats, ParOrb2, ParOut2, ILat,  $
			  Pressure2, PrevOrbILat, PARNAMES, IPar, PGrid
	    
            IF ParNames(IPar) EQ 'Temperature' THEN $
	          ParDiffOut = ParOut1 - ParOut2 ELSE $
              ParDiffOut = 100.0 * ((ParOut1 - ParOut2) / ParOut2)

            if(ThirdFile eq 1) then begin   ;third file to plot
               PressureGrid, PGridIn3, NPres, NOrbLats, ParOrb3, ParOut3, ILat,  $
			                 Pressure3, PrevOrbILat, PARNAMES, IPar, PGrid
            endif

;           do plotting on pressure levels
            PressurePlot, Logs(ipar), ParOut1, ParOut2, ParOut3, ParNames, IPar, Units, FileDescrip1,  $
                          FileDescrip2, FileDescrip3, ParMins, ParMaxs, IDX, PGrid, LatOrb,   $
			  LonOrb, ParDiffOut, dTitStr, dTitSym, DiffDatRange,  $
			  PTop, PBot, BegLat, EndLat, NDFiles, File1, File2, File3, ProfID,  $
			  ILat,PrevOrbILat, ParName1, ParName2, ThirdFile
	    
	  ENDIF ELSE BEGIN ;Plot on altitude vertical grid

;            plot on altitude levels
             AltitudePlot, Logs(ipar), ParOrb1, ParOrb2, ParOrb3, LatOrb, LonOrb, IDX, Zbot, Ztop,  $
                           ParNames, IPar, Units, FileDescrip1, FileDescrip2, FileDescrip3,  $
		           ParMins, ParMaxs, DiffDatRange, ParDiffOrb, dTitStr, $
			   dTitSym, BegLat, EndLat, NDFiles, File1, File2, File3, ProfID,  $
			   ILat,PrevOrbILat, ParName1, ParName2, ThirdFile
	  
	  ENDELSE

          IF !D.name EQ 'Z' THEN begin  ;this is to make png files
           ;; If a plot is complete write the image
           if ((!p.multi[0] eq 0) or (Ilat eq NLats-1)) then begin
           orbit_annotation = "_Orbits"+string(previous_orbit+1,format="(I2.2)")+"-"+string(NORBS, format="(I2.2)")
           imagefile= "OrbitPlot_"+strtrim(ParNames[ipar], 2)+"_Scan"+strtrim(NSCAN, 2)+orbit_annotation+".png"
           genimg_2, filename=imagefile
           previous_orbit = NORBS
           endif
          ENDIF

          PrevOrbILat = ILat

        ENDIF   ; End of IF for enough points in orbit to plot

      ENDIF ELSE BEGIN   ; invalid orbit

        ;PRINT, 'Orbit ' + STRTRIM(STRING(NOrbs), 1) + ' skipped.'
        ;PrevOrbILat = ILat
        PRINT, 'Plotted requested orbits.'
	EXIT

      ENDELSE  ;end of if for valid orbit

    ENDIF   ; End of IF for current orbit

  ENDFOR   ; End of FOR loop over latitudes
			       
ENDFOR   ; End of FOR loop over parameters

If StrUpCase(!d.name) EQ 'PS' Then Begin
  Device, /close
  Set_Plot, 'X'
  !P.MULTI = 0
EndIf

END   ;; MAIN 


